<?php

namespace Litepci\Todo\Http\Controllers;

use App\Http\Controllers\PublicController as BaseController;
use Litepci\Todo\Interfaces\TodoRepositoryInterface;

class TodoPublicController extends BaseController
{
    // use TodoWorkflow;

    /**
     * Constructor.
     *
     * @param type \Litepci\Todo\Interfaces\TodoRepositoryInterface $todo
     *
     * @return type
     */
    public function __construct(TodoRepositoryInterface $todo)
    {
        $this->repository = $todo;
        parent::__construct();
    }

    /**
     * Show todo's list.
     *
     * @param string $slug
     *
     * @return response
     */
    protected function index()
    {
        $todos = $this->repository
        ->pushCriteria(app('Litepie\Repository\Criteria\RequestCriteria'))
        ->scopeQuery(function($query){
            return $query->orderBy('id','DESC');
        })->paginate();


        return $this->response->setMetaTitle(trans('todo::todo.names'))
            ->view('todo::public.todo.index')
            ->data(compact('todos'))
            ->output();
    }


    /**
     * Show todo.
     *
     * @param string $slug
     *
     * @return response
     */
    protected function show($slug)
    {
        $todo = $this->repository->scopeQuery(function($query) use ($slug) {
            return $query->orderBy('id','DESC')
                         ->where('slug', $slug);
        })->first(['*']);

        return $this->response->setMetaTitle($todo->name . trans('todo::todo.name'))
            ->view('todo::public.todo.show')
            ->data(compact('todo'))
            ->output();
    }

}
