<?php

namespace Litepci\Todo\Policies;

use Litepie\User\Contracts\UserPolicy;
use Litepci\Todo\Models\Todo;

class TodoPolicy
{

    /**
     * Determine if the given user can view the todo.
     *
     * @param UserPolicy $user
     * @param Todo $todo
     *
     * @return bool
     */
    public function view(UserPolicy $user, Todo $todo)
    {
        if ($user->canDo('todo.todo.view') && $user->isAdmin()) {
            return true;
        }

        return $todo->user_id == user_id() && $todo->user_type == user_type();
    }

    /**
     * Determine if the given user can create a todo.
     *
     * @param UserPolicy $user
     * @param Todo $todo
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('todo.todo.create');
    }

    /**
     * Determine if the given user can update the given todo.
     *
     * @param UserPolicy $user
     * @param Todo $todo
     *
     * @return bool
     */
    public function update(UserPolicy $user, Todo $todo)
    {
        if ($user->canDo('todo.todo.edit') && $user->isAdmin()) {
            return true;
        }

        return $todo->user_id == user_id() && $todo->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given todo.
     *
     * @param UserPolicy $user
     * @param Todo $todo
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Todo $todo)
    {
        return $todo->user_id == user_id() && $todo->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given todo.
     *
     * @param UserPolicy $user
     * @param Todo $todo
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Todo $todo)
    {
        if ($user->canDo('todo.todo.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given todo.
     *
     * @param UserPolicy $user
     * @param Todo $todo
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Todo $todo)
    {
        if ($user->canDo('todo.todo.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
